import java.net.*;
import java.io.*;

public class WebServerApp {
 public static void main(String args[]){
  WebServer server = new WebServer();
  server.run();
 }
}
class WebServer extends Thread{
 public WebServer() {
  super();
 }
 public void run() {
  try{
   ServerSocket server = new ServerSocket(8080);
   int localPort = server.getLocalPort();
   System.out.println("WebServerApp is listening on port "+localPort+".");
   do {
    Socket client = server.accept();
    String destName = client.getInetAddress().getHostName();
    int destPort = client.getPort();
    System.out.println("Accepted connection to "+destName+
     " on port "+destPort+".");
    NVTOutputStream outStream = 
     new NVTOutputStream(client.getOutputStream());
    NVTInputStream inStream = 
     new NVTInputStream(client.getInputStream(),outStream);
    boolean finished = false;
    String inLine = inStream.readLine();
    System.out.println("Received: "+inLine);
    if(getRequest(inLine)) {
     String fileName = getFileName(inLine);
     File file = new File(fileName);
     if(file.exists()) {
      System.out.println(fileName+" requested.");
      outStream.println("HTTP/1.0 200 OK");
      outStream.println("MIME-Version: 1.0");
      outStream.println("Content-Type: text/html");
      int len = (int) file.length();
      outStream.println("Content-Length: "+len);
      outStream.println("");
      sendFile(outStream,file);
      outStream.flush();
     }else{
      outStream.println("HTTP/1.0 404 Not Found");
      String notFound ="<TITLE>Not Found</TITLE><H1>Error 404 - File Not Found</H1>";
      outStream.println("Content-Type: text/html");
      outStream.println("Content-Length: "+notFound.length()+2);
      outStream.println("");
      outStream.println(notFound);
      outStream.flush();
      }
    }
    try{
      sleep(1000);
    }catch(InterruptedException e){}    
    client.close();
   } while(true);
  }catch (IOException ex){
   System.out.println("IOException occurred.");
  }
 }
 boolean getRequest(String s) {
  if(s.length() > 0) {
   if(s.substring(0,3).equalsIgnoreCase("GET")) return true;
  }
  return false;
 }
 String getFileName(String s) {
  String f = s.substring(s.indexOf(' ')+1);
  f = f.substring(0,f.indexOf(' '));
  try {
   if(f.charAt(0) == '/') f =  f.substring(1);
  } catch(StringIndexOutOfBoundsException ex) {
  }
  if(f.equals("")) f = "index.html";
  return f;
 }
 void sendFile(NVTOutputStream out,File file) {
  try {
   DataInputStream in = new DataInputStream(new FileInputStream(file));
   int len = (int) file.length();
   byte buffer[] = new byte[len];
   in.readFully(buffer);
   out.write(buffer,0,len);
   in.close();
  }catch(Exception ex){
   System.out.println("Error retrieving file.");
   System.exit(1);
  }
 }
}

class NVTOutputStream extends PrintStream {
 int IAC = 255;
 byte CR = 13;
 byte LF = 10;
 public NVTOutputStream(OutputStream outStream) {
  super(outStream);
 }
 public void write(int i) {
  if(i == IAC) super.write(i);
  super.write(i);
 }
 public void println(String s) {
  super.print(s);
  super.write(CR);
  super.write(LF);
  super.flush();
 }
}

class NVTInputStream extends FilterInputStream {
 byte IAC = (byte) 0xff;
 byte DO = (byte) 0xfd;
 byte WILL = (byte) 0xfb;
 byte CR = 13;
 byte LF = 10;
 int WONT = 252;
 int DONT = 254;
 int BUFFER_SIZE = 1024;
 OutputStream out;
 byte lineBuffer[] = new byte[BUFFER_SIZE];
 int numBytes = 0;
 public NVTInputStream(InputStream inStream,OutputStream outStream) {
  super(inStream);
  out = outStream;
 }
 public int read() throws IOException {
  boolean recIAC;
  int i;
  do {
   recIAC = false;
   i = in.read();
   if(i == -1) return i;
   byte b = (byte) i;
   if(b == IAC) {
    recIAC = true;
    int cmd = in.read();
    if(cmd == -1) return cmd;
    byte b2 = (byte) cmd;
    if(b2 == IAC) return 255;
    else if(b2 == DO) {
     int opt = in.read();
     if(opt == -1) return opt;
     out.write(255);
     out.write(WONT);
     out.write(opt);
     out.flush();
    }else if(b2 == WILL) {
     int opt = in.read();
     if(opt == -1) return opt;
     out.write(255);
     out.write(DONT);
     out.write(opt);
     out.flush();
    }
   }
  } while(recIAC);
  return i;
 }
 public String readLine() throws IOException {
  numBytes = 0;
  boolean finished = false;
  do {
   int i = read();
   if(i == -1) return null;
   byte b = (byte) i;
   if(b == LF) {
    if(numBytes>0) {
     if(lineBuffer[numBytes-1] == 13) 
      return new String(lineBuffer,0,0,numBytes-1);
    }
   }
   lineBuffer[numBytes] = b;
   ++numBytes;
  } while (!finished);
  return null;
 }
}
